#!/usr/bin/perl

##########################################################
## Weiter nichts ndern ab hier
##########################################################

# CGI functions

sub parse_form {
# --------------------------------------------------------
# Extract the parameters and put it into a hash
  my(%in, @ffields, $input, $name, $value);
  
  if($ENV{'REQUEST_METHOD'} eq "POST") { read(STDIN, $input, $ENV{'CONTENT_LENGTH'}); }
  else { $input = $ENV{'QUERY_STRING'}; }
  @ffields = split(/&/, $input);
  for(@ffields) {
    ($name, $value) = split /=/;
    $value =~ tr/+/ /;
    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    if(!defined($in{$name})) { $in{$name} = $value; }
    else { $in{$name} .= "~~$value"; }
  }
  return %in;
}

sub html_print_headers {
# --------------------------------------------------------
# Print out the headers if they haven't already been printed.

	if(!$html_headers_printed) {
		print "Pragma: no-cache\n"  if($db_nocache);
		print "Content-type: text/html\n\n";
		$html_headers_printed = 1;
	}
}

sub cgierr {
# --------------------------------------------------------
# Displays any errors and prints out FORM and ENVIRONMENT 
# information. Useful for debugging.

	&html_print_headers;
	print "<pre>\nCGI Error: $!\nMessage: $_[0]\n\nFORM VARIABLES\n";
	for(sort keys %in) { print "$_: \t$in{$_}\n"; }
	print "\nENVIRONMENT VARIABLES\n";
	for(sort keys %ENV) {	print "$_: \t$ENV{$_}\n"; }
	print "</pre>";
	exit 1;
}


# Banners database functions

sub get_banners {
# --------------------------------------------------------
# Gets the URLs of all the banners
  @db_banners_url = ();
  open(BANNERS, '<'.$db_bannersdata_path) or &cgierr("Can't open banners database. Grund: $!");
  $db_user_flock and (flock(BANNERS, 1) or &cgierr("Can't get file lock on banners database ($db_bannersdata_path). Grund: $!"));
  while(<BANNERS>) {
    /\#/ and next;
    chomp;
    /^(.+?)\|(.+?)$/ and push(@db_banners_url, "$1|$2");
  }
  close BANNERS;
}

sub add_banner {
# --------------------------------------------------------
# adds the url of a new banner
  unless($in{'bannerurl'} =~ /[a-zA-Z]/ || $in{'bannerurl'} eq "http://") {
    &html_error("Incorrect banner url.") and return;
  }
  
  # Get ID
  &get_banners;
  $in{'bid'} = $db_banners_url[-1] + 1;
  
  # Save new banner
  open(BANNERS, ">>".$db_bannersdata_path) or &cgierr("Can't open banners database. Grund: $!");
  $db_user_flock and (flock(BANNERS, 2) or &cgierr("Can't get exclusive file lock on banners database ($db_bannersdata_path). Grund: $!"));
  print BANNERS "$in{'bid'}|$in{'bannerurl'}\n";
  close BANNERS;
  
  # success msg
  &html_addbanner_success;
}

sub delete_banner {
# --------------------------------------------------------
# deletes the url of a banner
  my($i, $gef);
  
  &get_banners;
  # Delete banner
  open(BANNERS, '>'.$db_bannersdata_path) or &cgierr("Can't open banners database.");
  $db_user_flock and (flock(BANNERS, 2) or &cgierr("Can't get exclusive file lock on banners database ($db_bannersdata_path). Grund: $!"));
  for(@db_banners_url) {
    ($bid, $burl) = split /\|/;
    if($bid != $in{'bannerid'}) { print BANNERS "$bid|$burl\n"; }
    else                        { $gef = 1; }
  }
  
  # Success or error msg
  ($gef) ?
    &html_deletebanner_success:
    &html_error("Banner id $in{'bannerid'} not found in database.");
}


# Clients database functions

sub get_clientsdata {
# --------------------------------------------------------
# Get all clients data
  @clientsdata = ();
  open(CLIENTS, '<'.$db_clientsdata_path) or &cgierr("Can't open Clients database");
  $db_user_flock and (flock(CLIENTS, 1) or &cgierr("Can't get file lock on clients database ($db_clientsdata_path). Grund: $!"));
  while(<CLIENTS>) {
    /\#/ and next;
    chomp;
    /^(.+?)$/ and push(@clientsdata, $1);
  }
  close CLIENTS;
}

sub user_select {
# --------------------------------------------------------
# Check for existence of user ID
  my($id) = shift;

  &get_clientsdata;
  for(@clientsdata) { /^$id\|/ and @userdata = split /\|/ and last; }
  
  return $userdata[0];
}

sub save_clientsdata {
# --------------------------------------------------------
# Saves actual userdata in clients database.
  my(@tmpdata, $found);
  
  &get_clientsdata;
  open(CLIENTS, '>'.$db_clientsdata_path) or &cgierr("Can't open clients database for writing. Grund: $!");
  $db_user_flock and (flock(CLIENTS, 2) or &cgierr("Can't get exclusive file lock on clients database ($db_clientsdata_path). Grund: $!"));
  for(@clientsdata) {
    @tmpdata = split /\|/;
    if($tmpdata[0] == $userdata[0]) { @tmpdata = @userdata;  $found++; }
    print CLIENTS join("|", @tmpdata), "\n"  if($#tmpdata > 1);
  }
  print CLIENTS join("|", @userdata), "\n"  if(!$found && $#userdata > 1);
  close CLIENTS;
}

sub view_clients {
# --------------------------------------------------------
# Prepares all clients data for displaying
  my(@tmpdata, $bcr, $bcp, $ccr, $ccp);
  local(@output, $totusers, $totearnings);
  
  &get_clientsdata;
  $totusers = 0;  $totearnings = "0.00";
  for(@clientsdata) {
    @tmpdata = split /\|/;
    $tmpdata[5] =~ s/~~/, /g;  # format banner ids
    $tmpdata[5] = "All"  if(!$tmpdata[5]);
    if($tmpdata[4]) { $tmpdata[4] = "Yes"; } else { $tmpdata[4] = "No"; } # Portal site?
    ($bcr, $bcp, $ccr, $ccp) = &calc_log($tmpdata[0]);  # count log files
    $tmpdata[7] += $bcr;  $tmpdata[8] += $bcp;  $tmpdata[9] += $ccr;  $tmpdata[10] += $ccp;
    if($tmpdata[8] != 0 && $tmpdata[10] != 0) {
      $tmpdata[11] = sprintf("%.1f", 100 / $tmpdata[8] * $tmpdata[10])."% ".int($tmpdata[8] / $tmpdata[10]).":1";  # Click per Impression
    } else { $tmpdata[11] = "0% 0:0"; }
    $tmpdata[12] = sprintf("%.2f", $db_paypic_factor * $tmpdata[8] + $db_payclick_factor * $tmpdata[10]);  # earnings
      
    push(@output, join("|", $tmpdata[1], $tmpdata[0], $tmpdata[3], $tmpdata[4], $tmpdata[5], $tmpdata[6], $tmpdata[11], $tmpdata[12]));
    $totusers++;  $totearnings = sprintf("%.2f", $totearnings + $tmpdata[12]);
  }
  @output = sort { $a cmp $b } @output;
  &html_view_clients;
}

sub show_client {
# --------------------------------------------------------
# Shows client details
  my($a, $b, $c, $d);
  &user_select($in{'id'}) or &html_error("Unknown user id");
  
  # Banners
  $userdata[5] = "~~".$userdata[5]."~~";
  &get_banners;
  if($userdata[5] =~ /~~0~~/) {
    $c = qq|<OPTION VALUE="0" SELECTED>All banners|;
  } else {
    $c = qq|<OPTION VALUE="0">All banners|;
  }
  for(@db_banners_url) {
    ($a, $b) = split /\|/;
    if($userdata[5] =~ /~~$a~~/) { $c .= qq|<OPTION VALUE="$a" SELECTED>$b|; }
    else { $c .= qq|<OPTION VALUE="$a">$b|; }
  }
  $userdata[5] = $c;
  
  # Portal?
  if($userdata[4]) {
    $userdata[4] = qq|<OPTION VALUE="1" SELECTED>Yes<OPTION VALUE="0">No|;
    $db_counteruse =~ s/<%CLICK%>/$db_cgibin_url\/portal.pl\/clickportal\?id=$userdata[0]/gi;
    $db_counteruse =~ s/<%BANNER%>/$db_cgibin_url\/portal.pl\/banner\?id=$userdata[0]/gi;
  }
  else {
    $userdata[4] = qq|<OPTION VALUE="1">Yes<OPTION VALUE="0" SELECTED>No|;
    $db_counteruse =~ s/<%CLICK%>/$db_cgibin_url\/count.pl\/click\?id=$userdata[0]/gi;
    $db_counteruse =~ s/<%BANNER%>/$db_cgibin_url\/count.pl\/banner\?id=$userdata[0]/gi;
  }
  $db_counteruse =~ s/</\&lt\;/g;  $db_counteruse =~ s/>/\&gt\;/g;  $db_counteruse =~ s/"/\&quot\;/g;
  
  # Count log files
  ($a, $b, $c, $d) = &calc_log($userdata[0]);
  $userdata[7] += $a;  $userdata[8] += $b;  $userdata[9] += $c;  $userdata[10] += $d;
  
  # Clicks per Impression
  if($userdata[8] != 0 && $userdata[10] != 0) {
    $userdata[11] = sprintf("%.1f", 100 / $userdata[8] * $userdata[10])."% ".int($userdata[8] / $userdata[10]).":1";
  } else { $userdata[11] = "0% 0:0"; }
  
  # Earnings
  $userdata[12] = sprintf("%.2f", $db_paypic_factor * $userdata[8] + $db_payclick_factor * $userdata[10]);
  
  &html_show_client;
}

sub add_client {
# --------------------------------------------------------
# Add a Client to the list
  my($id, $notfound, @chars);
  
  # Check received data
  unless($in{'clientname'} =~ /[a-zA-Z0-9]/) { &html_error("Incorrect name.") and return; }
  unless($in{'clientemail'} =~ /[a-zA-Z0-9]/) { &html_error("Incorrect email.") and return; }
  unless($in{'clientpwd1'} =~ /[a-zA-Z0-9]/ && length($in{'clientpwd1'}) < 10 && length($in{'clientpwd1'}) > 2) { &html_error("Incorrect Password.") and return; }
  unless($in{'clientpwd1'} eq $in{'clientpwd2'}) { &html_error("You didn't typed two times the same password.") and return; }
  unless($in{'clientdesturl'} =~ /[a-zA-Z0-9]/ && $in{'clientdesturl'} ne 'http://') { &html_error("Incorrect Destination URL.") and return; }
  unless($in{'clientbanner'} =~ /[0-9]/) { $in{'clientbanner'} = 0; }
  
  # Create user ID
  &get_clientsdata;
  srand;
  do {
    $in{'id'} = int(rand(899999)) + 100000;
    if(&user_select($in{'id'}) == 0) { $notfound = 1; }
  } until($notfound == 1);
  
  # Crypt Password
  @chars = ('a'..'z', 'A'..'Z', 0..9);
  $in{'pwd'} = crypt($in{'clientpwd1'}, $chars[rand(@chars)].$chars[rand(@chars)]);
  
  # Touch log files for new user
  open(NEWFILE, '>'.$db_data_path.'/'.$in{'id'}."_click") or &cgierr("Can't create click log file for new client.");
  close NEWFILE;
  open(NEWFILE, '>'.$db_data_path.'/'.$in{'id'}."_banner") or &cgierr("Can't create banner log file for new client.");
  close NEWFILE;
  chmod(0666, $db_data_path.'/'.$in{'id'}."_click", $db_data_path.'/'.$in{'id'}."_banner");
  
  # Save client
  @userdata = ($in{'id'}, $in{'clientname'}, $in{'pwd'}, $in{'clientemail'}, $in{'clientportal'}, $in{'clientbanner'}, $in{'clientdesturl'}, 0, 0, 0, 0);
  &save_clientsdata;
  
  # Print a success msg and the data out
  &show_client;
}

sub delete_clientdata {
# --------------------------------------------------------
# deletes a client
  my($file);
  
  # Check user id
  &user_select($in{'id'}) or (&html_error("User id doesn't exist.") and return);
  @userdata = ($in{'id'});
  &save_clientsdata;
  @userdata = ();
  
  # Delete log files
  unlink($db_data_path.'/'.$in{'id'}."_click", $db_data_path.'/'.$in{'id'}."_banner") or
    (&html_error("Can't delete client statistic.") and return);
  
  # Success msg
  &view_clients;
}

sub modify_client {
# --------------------------------------------------------
# modifies the client's data
  my($value, @tempdata, $input);
  
  # Check user id
  &user_select($in{'id'}) or (&html_error("Unknown user id.") and return);
  
  # Check modified data
  unless($in{'clientname'} =~ /[a-zA-Z0-9]/) { &html_error("Incorrect name.") and return; }
  unless($in{'clientemail'} =~ /[a-zA-Z0-9]/) { &html_error("Incorrect email.") and return; }
  if($in{'clientpwd1'} =~ /[a-zA-Z0-9]/) { 
    unless($in{'clientpwd1'} eq $in{'clientpwd2'} && length($in{'clientpwd1'}) < 10 && length($in{'clientpwd1'}) > 2) {
      &html_error("Incorrect password.") and return;
    }
  }
  unless($in{'clientdesturl'} =~ /[a-zA-Z0-9]/ && $in{'clientdesturl'} ne 'http://') { &html_error("Incorrect Destination URL.") and return; }
  unless($in{'clientbanner'} =~ /[0-9]/) { $in{'clientbanner'} = 0; }
  
  # Password
  if($in{'clientpwd1'}) {
    @chars = ('a'..'z', 'A'..'Z', 0..9);
    $in{'pwd'} = crypt($in{'clientpwd1'}, $chars[rand(@chars)].$chars[rand(@chars)]);
  }
  else { $in{'pwd'} = $userdata[2]; }
  
  # Save changes
  @userdata = ($in{'id'}, $in{'clientname'}, $in{'pwd'}, $in{'clientemail'}, $in{'clientportal'}, $in{'clientbanner'}, $in{'clientdesturl'}, $userdata[7], $userdata[8], $userdata[9], $userdata[10]);
  &save_clientsdata;
    
  # Success msg
  &show_client;
}

sub db_update {
# --------------------------------------------------------
# Database update
  my($cntinfo, $prtinfo, @tmpdata, @banners, $bid, $burl);
  
  # Get banners and all clients
  &get_banners;
  &get_clientsdata;
  
  # create database
  for(@clientsdata) {
    @tmpdata = split /\|/;
    $tmpdata[5] = "~~".$tmpdata[5]."~~";
    @banners = ();
    for(@db_banners_url) {
      ($bid, $burl) = split /\|/;
      push(@banners, $burl)  if($tmpdata[5] =~ /~~$bid~~/ || $tmpdata[5] =~ /~~0~~/);
    }
    ($tmpdata[4]) ?
      ($prtinfo .= join("|", $tmpdata[0], $tmpdata[6], @banners)."\n"):
      ($cntinfo .= join("|", $tmpdata[0], $tmpdata[6], @banners)."\n");
  }
  
  # save content
  open(CNTINFO, '>'.$db_cntinfo_path) or &cgierr("Can't open counter database. Grund: $!");
  $db_user_flock and (flock(CNTINFO, 2) or &cgierr("Can't get exclusive file lock on counter database. Grund: $!"));
  print CNTINFO $cntinfo;
  close CNTINFO;
  open(PRTINFO, '>'.$db_prtinfo_path) or &cgierr("Can't open portal counter database. Grund: $!");
  $db_user_flock and (flock(PRTINFO, 2) or &cgierr("Can't get exclusive file lock on portal counter database. Grund: $!"));
  print PRTINFO $prtinfo;
  close PRTINFO;
  
  &html_update_success;
}


# log files

sub client_log {
# --------------------------------------------------------
# Displays the clients' log files:
  my($output, $ip, $time, $host, %data);
  use IO::Socket;
  
  # Verify client id
  &user_select($in{'id'}) or (&html_error("Unknown User id.") and return);
  
  # open file
  if($in{'clients_log'} == 1) {
    (open(CLIENT, '<'.$db_data_path.'/'.$in{'id'}."_banner") or &cgierr("Can't open client's log file. Grund: $!"));
  } else {
    (open(CLIENT, '<'.$db_data_path.'/'.$in{'id'}."_click") or &cgierr("Can't open client's log file. Grund: $!"));
  }
  $db_user_flock and (flock(CLIENT, 1) or &cgierr("Can't get file lock on client's log file. Grund: $!"));
  while(<CLIENT>) {
    /^\n$/ and next;
    chomp;
    ($ip, $time) = split /\|/;
    $host = gethostbyaddr(inet_aton($ip), AF_INET) || $ip;
    unless($time <= $data{$ip} + $db_cheatprot_tolval) {
      $output .= &format_time($time).qq| - $ip - $host\n|;
    } else {
      $output .= qq|<EM>|.&format_time($time).qq| - $ip - $host</EM>\n|;
    }
    $data{$ip} = $time;
  }
  close CLIENT;
  
  &html_clientlog_show($output);
}

sub calc_log {
# --------------------------------------------------------
# Calculates counter
  my($id) = shift;
  my($bcr, $bcp, $ccr, $ccp, %data, $ip, $time);
  
  open(CLIENT, '<'.$db_data_path.'/'.$id."_banner") or &cgierr("Can't open client log file. Grund: $!");
  $db_user_flock and (flock(CLIENT, 1) or &cgierr("Can't get file lock on client log file. Grund: $!"));
  while(<CLIENT>) {
    /^\n$/ and next;
    chomp;
    ($ip, $time) = split /\|/;
    $bcr++;  $bcp++;
    $bcp--  if($time <= $data{$ip} + $db_cheatprot_tolval);
    $data{$ip} = $time;
  }
  close CLIENT;
  %data = ();
  open(CLIENT, '<'.$db_data_path.'/'.$id."_click") or &cgierr("Can't open client log file. Grund: $!");
  $db_user_flock and (flock(CLIENT, 1) or &cgierr("Can't get file lock on client log file. Grund: $!"));
  while(<CLIENT>) {
    /^\n$/ and next;
    chomp;
    ($ip, $time) = split /\|/;
    $ccr++;  $ccp++;
    $ccp--  if($time <= $data{$ip} + $db_cheatprot_tolval);
    $data{$ip} = $time;
  }
  close CLIENT;
  
  return ($bcr, $bcp, $ccr, $ccp);
}

sub counters_reset {
# --------------------------------------------------------
# deletes contents of log files and set counters to 0.
  &user_select($in{'id'}) or (&html_error("Unknown user id.") and return);
  
  # Save new counters
  $userdata[7] = $userdata[8] = $userdata[9] = $userdata[10] = 0;
  &save_clientsdata;
  
  # Remove all logs
  open(CLIENT, '>'.$db_data_path."/$in{'id'}_click") or &cgierr("Can't open client log file. Grund: $!");
  $db_user_flock and (flock(CLIENT, 2) or &cgierr("Can't get file lock on client log file. Grund: $!"));
  print CLIENT "";  close CLIENT;
  open(CLIENT, '>'.$db_data_path."/$in{'id'}_banner") or &cgierr("Can't open client log file. Grund: $!");
  $db_user_flock and (flock(CLIENT, 2) or &cgierr("Can't get file lock on client log file. Grund: $!"));
  print CLIENT "";  close CLIENT;
  
  &show_client;
}


##########################################################
## Client functions                                     ##
##########################################################

sub show_details {
# --------------------------------------------------------
# shows client details (counter, ...)
  my($bid, $burl, $bcr, $bcp, $ccr, $ccp);
  local(@banners, $earnings);
  
  # Verify user
  &user_select($in{'id'}) or (&html_showdetails_error("User not found.") and return);
  &html_showdetails_error("User not found.") and return  if
    (crypt($in{'pwd'}, substr($userdata[2], 0, 2)) ne $userdata[2]);
  
  # Calculate click counter
  ($bcr, $bcp, $ccr, $ccp) = &calc_log($in{'id'});
  $userdata[7] += $bcr;  $userdata[8] += $bcp;  $userdata[9] += $ccr;  $userdata[10] += $ccp;
  
  # Get Banner urls
  &get_banners;
  $userdata[5] = "~~$userdata[5]~~";
  for(@db_banners_url) {
    ($bid, $burl) = split /\|/;
    push(@banners, $burl)  if($userdata[5] =~ /~~$bid~~/ || $userdata[5] eq "~~0~~");
  }
  
  # How much did client earn?
  $earnings = sprintf("%.2f", $db_payclick_factor * $userdata[10] + $db_paypic_factor * $userdata[8]);
  
  # Code to paste
  if($userdata[4]) {
    $db_counteruse =~ s/<%CLICK%>/$db_cgibin_url\/portal.pl\/clickportal\?id=$userdata[0]/gi;
    $db_counteruse =~ s/<%BANNER%>/$db_cgibin_url\/portal.pl\/banner\?id=$userdata[0]/gi;
  }
  else {
    $db_counteruse =~ s/<%CLICK%>/$db_cgibin_url\/count.pl\/click\?id=$userdata[0]/gi;
    $db_counteruse =~ s/<%BANNER%>/$db_cgibin_url\/count.pl\/banner\?id=$userdata[0]/gi;
  }
  $db_counteruse =~ s/</\&lt\;/g;  $db_counteruse =~ s/>/\&gt\;/g;  $db_counteruse =~ s/"/\&quot\;/g;
  
  # Show Details
  &html_showdetails_success($earnings, @banners);
}

sub change_pwd {
# --------------------------------------------------------
# changes pwd
  my(@tmpdata);
  
  # Check user id
  &user_select($in{'id'}) or (&html_changepwd_error("Unknown user id.") and return);

  # Check old pwd
  if(crypt($in{'oldpwd'}, substr($userdata[2], 0, 2)) ne $userdata[2]) {
    &html_changepwd_error("Old password was incorrect.") and return;
  }
  
  # Check if new pwd1 is equal to pwd2
  (&html_changepwd_error("Incorrect new password.") and return)
    if($in{'newpwd1'} !~ /[a-zA-Z0-9]/ || length($in{'newpwd1'}) > 9 || length($in{'newpwd1'}) < 3);
  if($in{'newpwd1'} ne $in{'newpwd2'}) {
    &html_changepwd_error("Incorrectly retyped new password.") and return;
  }
  
  # Crypt new password
  $userdata[2] = crypt($in{'newpwd1'}, substr($userdata[2], 0, 2));
  
  # Save new password
  &save_clientsdata;

  # success msg
  &html_changepwd_success;
}

1;